@@ -10,7 +10,7 @@ api = TikTokApi.get_instance()
 # need to pass it to methods you want to download
 device_id = api.generate_device_id()
 
-trending = api.trending(custom_device_id=device_id)
+trending = api.by_trending(custom_device_id=device_id)
 
 # Below is if the method used if you have the full tiktok object
 video_bytes = api.get_video_by_tiktok(trending[0], custom_device_id=device_id)
